package inf101.v18.rogue101.items;

import inf101.v18.rogue101.game.IGame;
import inf101.v18.rogue101.objects.IItem;

import java.util.ArrayList;
import java.util.List;

public class Backpack<T extends IItem> implements IContainer {
    /**
     * A list containing everything in the backpack.
     */
    private List<T> content = new ArrayList<>();
    /**
     * The maximum amount of items allowed in a single backpack.
     */
    private final int MAX_SIZE = 5;

    @Override
    public int getCurrentHealth() {
        return 0;
    }

    @Override
    public int getDefence() {
        return 0;
    }

    @Override
    public int getMaxHealth() {
        return 0;
    }

    @Override
    public String getName() {
        return "Backpack";
    }

    @Override
    public int getSize() {
        return 20;
    }

    @Override
    public String getSymbol() {
        return "B";
    }

    @Override
    public int handleDamage(IGame game, IItem source, int amount) {
        return 0;
    }

    /**
     * Retrieves the current size of the Backpack.
     *
     * @return
     */
    public int size() {
        return content.size();
    }

    /**
     * Checks if the Backpack is empty
     *
     * @return  True if empty. False otherwise
     */
    public boolean isEmpty() {
        return content.isEmpty();
    }

    /**
     * Tries to add an item to the Backpack
     *
     * @param item  The item to add
     * @return
     */
    public boolean add(T item) {
        if (content.size() < MAX_SIZE) {
            content.add(item);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Removes an element at index i
     *
     * @param i The index of an element
     */
    public void remove(int i) {
        content.remove(i);
    }

    /**
     * Gets a T at index i,
     *
     * @param i The index of an element
     * @return  An object of type T
     */
    public T get(int i) {
        return content.get(i);
    }

    /**
     * Gets the content List for direct manipulation.
     *
     * @return  A list of T
     */
    public List<T> getContent() {
        return content;
    }

    @Override
    public boolean isFull() {
        if (content.size() >= MAX_SIZE) {
            return true;
        } else {
            return false;
        }
    }

    //This shows as an error, but is necessary to compile.
    @Override
    public int compareTo(Object o) {
        return compareTo((IItem)o);
    }
}
